#include "../pikascript-lib/PikaStdDevice/pika_hal.h"
#include "bflb_uart.h"

typedef struct platform_uart_t {
    struct bflb_device_s* device;
    struct bflb_uart_config_s config;
} platform_uart_t;

int pika_hal_platform_UART_open(pika_dev* dev, char* name) {
    if (name[0] == 'U' && name[1] == 'A' && name[2] == 'R' && name[3] == 'T') {
        int UART_num = fast_atoi(name + 4);
        platform_uart_t* uart = pikaMalloc(sizeof(platform_uart_t));
        uart->device = bflb_device_get_by_id(BFLB_DEVICE_TYPE_UART, UART_num);
        dev->platform_data = uart;
        return 0;
    }
    return -1;
}

int pika_hal_platform_UART_close(pika_dev* dev) {
    pikaFree(dev->platform_data, sizeof(platform_uart_t));
    return 0;
}

static void _uart_irq_handler(int irq, void* arg) {
    pika_dev* dev = (pika_dev*)arg;
    platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
    pika_hal_UART_config* cfg = (pika_hal_UART_config*)dev->ioctl_config;
    uint32_t intstatus = bflb_uart_get_intstatus(uart->device);
    if (intstatus) {
        cfg->event_callback(dev, cfg->event_callback_filter);
        bflb_uart_int_clear(uart->device, intstatus);
    }
}

int pika_hal_platform_UART_ioctl_config(pika_dev* dev,
                                        pika_hal_UART_config* cfg) {
    platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
    uart->config.baudrate = cfg->baudrate;
    switch (cfg->data_bits) {
        case PIKA_HAL_UART_DATA_BITS_5:
            uart->config.data_bits = UART_DATA_BITS_5;
            break;
        case PIKA_HAL_UART_DATA_BITS_6:
            uart->config.data_bits = UART_DATA_BITS_6;
            break;
        case PIKA_HAL_UART_DATA_BITS_7:
            uart->config.data_bits = UART_DATA_BITS_7;
            break;
        case PIKA_HAL_UART_DATA_BITS_8:
            uart->config.data_bits = UART_DATA_BITS_8;
            break;
        default:
            uart->config.data_bits = UART_DATA_BITS_8;
            break;
    }
    switch (cfg->parity) {
        case PIKA_HAL_UART_PARITY_NONE:
            uart->config.parity = UART_PARITY_NONE;
            break;
        case PIKA_HAL_UART_PARITY_ODD:
            uart->config.parity = UART_PARITY_ODD;
            break;
        case PIKA_HAL_UART_PARITY_EVEN:
            uart->config.parity = UART_PARITY_EVEN;
            break;
        default:
            uart->config.parity = UART_PARITY_NONE;
            break;
    }
    switch (cfg->stop_bits) {
        case PIKA_HAL_UART_STOP_BITS_1:
            uart->config.stop_bits = UART_STOP_BITS_1;
            break;
        case PIKA_HAL_UART_STOP_BITS_2:
            uart->config.stop_bits = UART_STOP_BITS_2;
            break;
        case PIKA_HAL_UART_STOP_BITS_1_5:
            uart->config.stop_bits = UART_STOP_BITS_1_5;
            break;
        default:
            uart->config.stop_bits = UART_STOP_BITS_1;
            break;
    }

    switch (cfg->flow_control) {
        case PIKA_HAL_UART_FLOW_CONTROL_NONE:
            uart->config.flow_ctrl = UART_FLOWCTRL_NONE;
            break;
        case PIKA_HAL_UART_FLOW_CONTROL_CTS:
            uart->config.flow_ctrl = UART_FLOWCTRL_CTS;
            break;
        case PIKA_HAL_UART_FLOW_CONTROL_RTS:
            uart->config.flow_ctrl = UART_FLOWCTRL_RTS;
            break;
        case PIKA_HAL_UART_FLOW_CONTROL_RTS_CTS:
            uart->config.flow_ctrl = UART_FLOWCTRL_RTS_CTS;
            break;
        default:
            uart->config.flow_ctrl = UART_FLOWCTRL_NONE;
            break;
    }

    /* support event callback */
    if (NULL != cfg->event_callback &&
        PIKA_HAL_EVENT_CALLBACK_ENA_ENABLE == cfg->event_callback_ena) {
        switch (cfg->event_callback_filter) {
            /* Configure UART to interrupt mode */
            case PIKA_HAL_UART_EVENT_SIGNAL_RX:
                bflb_uart_rxint_mask(uart->device, false);
                break;
            case PIKA_HAL_UART_EVENT_SIGNAL_TX:
                bflb_uart_txint_mask(uart->device, false);
                break;
            default:
                __platform_printf(
                    "Error: not supported event callback filter %d\r\n",
                    cfg->event_callback_filter);
                return -1;
        }
        bflb_irq_attach(uart->device->irq_num, _uart_irq_handler, dev);
        bflb_irq_enable(uart->device->irq_num);
    }
    return 0;
}

int pika_hal_platform_UART_ioctl_enable(pika_dev* dev) {
    if (!dev->is_enabled) {
        platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
        bflb_uart_init(uart->device, &uart->config);
        return 0;
    }
    return -1;
}

int pika_hal_platform_UART_ioctl_disable(pika_dev* dev) {
    if (dev->is_enabled) {
        platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
        bflb_uart_deinit(uart->device);
        return 0;
    }
    return -1;
}

int pika_hal_platform_UART_write(pika_dev* dev, void* buf, size_t count) {
    platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
    return bflb_uart_put(uart->device, buf, count);
}

int pika_hal_platform_UART_read(pika_dev* dev, void* buf, size_t count) {
    platform_uart_t* uart = (platform_uart_t*)dev->platform_data;
    return bflb_uart_get(uart->device, buf, count);
}
